package backend;

import intermediate.frame.*;
import java.util.*;
import intermediate.Temp;

/**
 * Ein Aktivierungssegment fr den Compiler
 */
public class I386Frame extends Frame {

    private Temp fp = new Temp("fp");
    private int numberOfFields, numberOfConsts;
    private ArrayList fields;

    /**
     * Leerer Konstruktor. Frames, die auf diese Art erzeugt werden, knnen nicht im ausfhrbaren
     * Programm verwendet werden. Sie dienen dazu, um im Zwischencode auf Werte wie wordSize() oder
     * das Temp fp zugreifen zu knnen.
     */
    public I386Frame() {
    }

    /**
     * Konstruktor
     * @param numberOfFields Anzahl der Felder: reservierte, Konstanten und Variablen
     * @param numberOfConsts Anzahl der Konstanten
     * @param consts Werte der Konstanten
     */
    public I386Frame(int numberOfFields, int numberOfConsts, List consts) {
        this.numberOfFields = numberOfFields;
        this.numberOfConsts = numberOfConsts;
        fields = new ArrayList(numberOfFields + getMinOffset());
        for (int i = 0; i < numberOfFields + getMinOffset(); i++) {
            fields.add(i, new Integer(0));
        }
        for (int i = 0; i < numberOfConsts; i++) {
            fields.set(i + getMinOffset(), consts.get(i));
        }
    }

    public int wordSize() {
        return 4;
    }

    public Temp fp() {
        return fp;
    }

    /**
     * Anzahl der Felder (Variablen und Konstant, OHNE die reservierten Felder!)
     * @return
     */
    public int getNumberOfFields() {
        return numberOfFields;
    }

    public int getNumberOfConsts() {
        return numberOfConsts;
    }

    /*public int[] getFields() {
        return fields;
    }*/
    public ArrayList getFields() {
        return fields;
    }

    public int getField(int i) {
        return ((Integer)fields.get(i)).intValue();
    }

    public void setField(int i, int value) {
        fields.set(i, new Integer(value));
    }



    /**
     * Erzeugt eine 1:1 Kopie des aktuellen Frames, da bei Rekursion im erzeugten Programm
     * immer eine neue Instanz verwendet werden muss. Eventuell belegte Variablenwerte werden
     * mit kopiert, dies lsst sich verhindern, indem man nur die Vorlage aus dem
     * FrameContainer zum Klonen nutzt. Dementsprechend darf das Frame aus dem
     * FrameContainer nicht verndert werden, sondern nur die erzeugten Kopien.
     */
    public Frame createClone() {
        I386Frame iframe = new I386Frame(numberOfFields, numberOfConsts, (List)null);
        ArrayList newList = new ArrayList();
        for (Iterator it = fields.iterator(); it.hasNext();) {
            newList.add(it.next());
        }
        iframe.fields = newList;
        return iframe;
    }

    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append("[");
        for (int i = 0; i < getFields().size(); i++) {
            sb.append((i == 0 ? "" : ",") + getFields().get(i));
        }
        sb.append("]");
        return sb.toString();
    }

}
