package frontend;

import compiler.*;
import java.util.Iterator;
import frontend.parser.*;
import frontend.symbol.*;
import java.io.*;
import antlr.*;
import frontend.parsetree.ProcedureNode;
import intermediate.CodeContainer;


public class PL0Frontend implements Frontend {

    private static compiler.Compiler compiler;
    private String filename;
    private PL0Parser parser;
    private boolean exceptionsfound;

    /**
     * Konstruktor, der erzeugende Compiler wird als Referenz bergeben und gespeichert.
     * @param compiler
     */
    public PL0Frontend(compiler.Compiler compiler) {
        this.compiler = compiler;
    }

    /**
     * Leerer Konstruktor, denn manchmal muss ein PL0Frontend erzeugt werden, ohne dass der Compiler
     * bekannt ist. Z.B. in IdentNode mchte man den Compiler ber PL0Frontend erst in Erfahrung bringen.
     */
    public PL0Frontend() {
    }


    /**
     * Liefert den zugehrigen Compiler zurck
     */
    public compiler.Compiler getCompiler() {
        return compiler;
    }

    /**
     * Setzt die Eingabequelle fr das Frontend. Muss ein Dateiname (String) sein.
     */
    public void setInputSource(Object source) {
        this.filename = (String)source;
    }

    /**
     * Transformiert die Eingabe in Zwischencode
     */
    public CodeContainer transformToIntermediate() {
        //Reader fr Eingabedatei erzeugen
        BufferedReader fileReader = null;
        try {
            fileReader = new BufferedReader(new FileReader(filename));
        }
        catch (FileNotFoundException ex) {
            System.err.println("File not found");
        }
        //Parsen
        parse(fileReader);
        //Wenn es keine Exceptions gab, AST bauen...
        if (!exceptionsfound) {
            ProcedureNode ast = (ProcedureNode)parser.getAST();
            //...und daraus den Zwischencode
            return ast.createIntermediateCode();
        }
        return null;
    }

    private void parse(BufferedReader input) {
        exceptionsfound = false;
        SymbolTable.getSymbolTable().reset();
        PL0Lexer lexer = new PL0Lexer(input);
        parser = new PL0Parser(lexer);
        //Beginne Parsen bei Startregel (program)
        try {
            parser.program();
        }
        //fange ANTLR Exceptions beim Scannen oder Parsen und gib sie aus
        catch (RecognitionException ex) {
            String errormsg = "Error at line " + ex.getLine() + ":" + ex.getColumn() +
                    " in file " + filename + ": " + ex.getMessage();
            System.err.println(errormsg);
            exceptionsfound = true;
        }

        catch (TokenStreamException ex) {
            //Fehlermeldung ist Standardmeldung der Exception
            String errormsg = ex.toString();
            //Wenn Exception TokenStreamRecognitionException ist, knnen wir die Fehlermeldung
            //in das Format unseres Parsers bringen.
            if (ex instanceof TokenStreamRecognitionException) {
                RecognitionException rex = ((TokenStreamRecognitionException)ex).recog;
                errormsg = "Error at line " + rex.getLine() + ":" + rex.getColumn() +
                    " in file " + filename + ": " + rex.getMessage();
            }
            System.err.println(errormsg);
            exceptionsfound = true;
        }

        //Wenn es keine ANTLR Exceptions gab, gib Exceptions der Symboltabelle aus.
        /*if (!exceptionsfound && parser.getExceptions().size() > 0) {
            for (Iterator it = parser.getExceptions().iterator(); it.hasNext(); ) {
                System.err.println((String)it.next());
            }
            exceptionsfound = true;
        }*/
    }


    public boolean exceptionsFound() {
        return exceptionsfound;
    }

}