package frontend.parsetree;

import intermediate.*;
import intermediate.operators.*;
import antlr.Token;

public class IfNode extends PL0Node {

    public IfNode(Token tok) {
        super(tok);
    }

    /**
     * Konvertiert den Knoten in ein Statement. Die Umwandlung erfolgt dabei so:
     *
     * Start:
     * if (condNode) then execNode;
     *
     * Die Knoten condNode und execNode werden in die Statements condStm und execStm umgewandelt.
     *
     * Ziel (linearisiert):
     * CJUMP(rel, par1, par2, t, f)   //rel, par1 und par2 sind der Vergleichsoperator und
     *                                //die zu vergleichenden Ausdrcke (gewonnen aus condStm)
     * LABEL(t)
     * execStm
     * JUMP(f)
     * LABEL(f)
     */
    public Statement convertStatement() {
        Label t = new Label();
        Label f = new Label();
        PL0Node condNode = (PL0Node)getFirstChild();
        PL0Node execNode = (PL0Node)condNode.getNextSibling();
        Statement condStm = condNode.convertConditional(t, f);
        Statement execStm = execNode.convertStatement();
        return new SEQ(condStm, new SEQ(new LABEL(t), new SEQ(execStm, new SEQ(new JUMP(f),
                new LABEL(f)))));
    };

}
