package frontend.parsetree;

import intermediate.*;
import java.util.*;
import intermediate.operators.*;
import antlr.Token;
import frontend.symbol.*;

public class ProcedureNode extends PL0Node {

    private SymbolTable table = SymbolTable.getSymbolTable();
    private static CodeContainer cc;
    private static FrameContainer fc;

    public ProcedureNode() {
        super();
    }

    public ProcedureNode(Token tok) {
        super(tok);
    }

    public CodeContainer createIntermediateCode() {
        cc = new CodeContainer();
        PL0Node node = (PL0Node)getFirstChild();
        createAll(node, 0);
        return cc;
    }

    /**
     * Erzeugt rekursiv die Syntaxbume fr alle Gltigkeitsbereiche. Alle erzeugten Bume
     * werden linear im sogenannten CodeContainer (HashMap) gespeichert. Dabei entspricht der Key
     * der HashMap der Reihenfolge des Auftretens des Gltigkeitsbereichs im Quellcode. Diese
     * Reihenfolge wurde in der Symboltabelle gespeichert und wird nun ausgelesen.
     */
    private void createAll(PL0Node node, int number) {
        //Prozedurnamen berspringen, der ist fr den Zwischencode irrelevant.
        //Da der erste Knoten nur in Prozeduren ein IdentNode ist, aber nicht
        //im Hauptprogramm, muss vor dem berspringen auf IdentNode getestet werden.
        if (node instanceof IdentNode) {
            node = (PL0Node)node.getNextSibling();
        }

        while (node instanceof ProcedureNode) {
            String procname = node.getFirstChild().getText();
            ScopeDef sd = (ScopeDef)table.lookup(procname);
            //Gltigkeitshierarchie nachbauen, damit die Variablen korrekt in Speicherberechnungen
            //umgewandelt werden (Folgen des Static Link)
            table.pushScope(sd);
            createAll((PL0Node)node.getFirstChild(), sd.getScopeNumber());
            table.popScope();
            node = (PL0Node)node.getNextSibling();
        }
        if (node != null) {
            cc.addCode(node.convertStatement(), number);
        }

    }



    /**
     * Erzeugt rekursiv CodeContainer, indem alle Prozeduren einer Prozedur (inkl Hauptprogramm)
     * und der Prozedurkrper (Code) zum CodeContainer hinzugefgt werden.
     */
    /*public CodeContainer createIntermediateCode() {
        CodeContainer cc = new CodeContainer("cc");
        PL0Node node = (PL0Node)getFirstChild();
        //Befinden wir uns im Knoten des Hauptprogrammes, ist node bereits der erste Prozedurknoten
        //oder der Begin-Knoten. Befinden wir uns aber in einem Prozedurknoten, kommt vorher
        //ein Ident-Knoten mit dem Namen der Prozedur.
        if (node instanceof IdentNode) {
            cc = new CodeContainer(node.getText()); //CodeContainer mit Prozedurnamen erzeugen
            node = (PL0Node)node.getNextSibling();  //weiterschalten zu Prozedur- oder Begin-Knoten
        } else {
            cc = new CodeContainer(getText());      //CodeContainer mit Namen "PROGRAM" erzeugen
        }
        //Wenn es weitere Geschwisterknoten gibt, mssen das Prozedurknoten sein, denn Begin-Knoten
        //kommen immer ganz am Ende und haben keine weiteren Geschwister => Rekursion starten
        //Es werden alle Prozedurknoten einer Prozedur der Reihe nach abgearbeitet.
        while (node.getNextSibling() != null) {
            String procname = node.getFirstChild().getText(); //Namen der Prozedur finden
            ScopeDef sd = (ScopeDef)table.lookup(procname);   //Prozedur aus Symboltabelle holen
            table.pushScope(sd);                              //und auf den Stack legen
            cc.addProcedure(((ProcedureNode)node).createIntermediateCode()); //Rekursion
            table.popScope();                                  //Prozedur vom Stack nehmen
            node = (PL0Node)node.getNextSibling();             //bearbeite nchstes Geschwister
        }
        //Letzter Knoten ist Begin-Knoten => in Statement umwandeln und zum CodeContainer hinzufgen
        cc.addCode(node.convertStatement());
        return cc;
    }*/

}
