package intermediate.operators;

import intermediate.*;

public class CJUMP extends Statement {

    public static final int EQ = 0; // =
    public static final int NE = 1; // !=
    public static final int LT = 2; // <
    public static final int GT = 3; // >
    public static final int LE = 4; // <=
    public static final int GE = 5; // >=

    private int relop;
    private Expression left, right;
    private Label iftrue, iffalse;

    public CJUMP(int relop, Expression left, Expression right, Label iftrue, Label iffalse) {
        this.relop = relop;
        this.left = left;
        this.right = right;
        this.iftrue = iftrue;
        this.iffalse = iffalse;
        addChild(left);
        addChild(right);
    }

    public String toString() {
        return "CJUMP(" + getRelop() + ", " + iftrue + ", " + iffalse + ")";
    }

    public String toStringList() {
        return "CJUMP(" + getRelop() + ", " + left.toStringList() + ", " + right.toStringList()
               + ", " + iftrue + ", " + iffalse + ")";
    }

    private String getRelop() {
        String op = null;
        switch (relop) {
            case (EQ): op = "=="; break;
            case (NE): op = "!="; break;
            case (LT): op = "<"; break;
            case (GT): op = ">"; break;
            case (LE): op = "<="; break;
            case (GE): op = ">="; break;
        }
        return op;
    }

    public int getRelopInt(){
    	return relop;
    }

    public Label getTrueLabel(){
    	return iftrue;
    }
    public Label getFalseLabel(){
    	return iffalse;
    }

    public Expression getLeftExp() {
      return left;
    }

    public Expression getRightExp() {
      return right;
    }

}
