package intermediate.visualize;

import intermediate.TreeNode;
import javax.swing.tree.*;
import javax.swing.event.*;
import javax.swing.*;

public class JTreeIntermediateModel implements TreeModel {

    private TreeNode root;
    private EventListenerList listenerList = new EventListenerList();

    public JTreeIntermediateModel(TreeNode root) {
        this.root = root;
    }

    /**
     * Setzt eine neue TreeNode als Wurzel und ruft fireRootChanged() auf
     */
    public void setRoot(TreeNode root) {
        this.root = root;
        fireRootChanged();
    }

    /**
     * Liefert das Kind mit Index index relativ zum aktuellen Knoten zurck
     */
    public Object getChild(Object parent, int index) {
        if (parent == null) {
            return null;
        }
        TreeNode p = (TreeNode)parent;
        TreeNode c = p.getFirstChild();
        if (c == null) {
            throw new ArrayIndexOutOfBoundsException("node has no children");
        }
        int i = 0;
        while (c != null && i < index) {
            c = c.getNextSibling();
            i++;
        }
        return c;
    }

    /**
     * Liefert die Anzahl der Kinder eines Knotens zurck
     */
    public int getChildCount(Object parent) {
        if (parent == null) {
            throw new IllegalArgumentException("root is null");
        }
        TreeNode p = (TreeNode)parent;
        return p.getNumberOfChildren();
    }

    /**
     * Liefert den Index eines bestimmten Kindes eines bestimmten Knotens zurck
     */
    public int getIndexOfChild(Object parent, Object child) {
        if (parent == null || child == null) {
            throw new IllegalArgumentException("root or child is null");
        }
        TreeNode p = (TreeNode)parent;
        TreeNode c = p.getFirstChild();
        if (c == null) {
            throw new ArrayIndexOutOfBoundsException("node has no children");
        }
        int i = 0;
        while (c != null && c != child) {
            c = c.getNextSibling();
            i++;
        }
        if (c == child) {
            return i;
        }
        throw new java.util.NoSuchElementException("node is not a child");

    }

    /**
     * Liefert die aktuelle Wurzel zurck
     */
    public Object getRoot() {
        return root;
    }

    /**
     * Gibt an, ob der bergebene Knoten ein Blatt ist (ob er keine weiteren Kinder hat)
     */
    public boolean isLeaf(Object node) {
        TreeNode n = (TreeNode)node;
        return n.getNumberOfChildren() == 0;
    }

    /**
     * Fgt einen TreeModelListener hinzu
     */
    public void addTreeModelListener(TreeModelListener l) {
        listenerList.add(TreeModelListener.class, l);
    }

    /**
     * Entfernt einen TreeModelListener
     */
    public void removeTreeModelListener(TreeModelListener l) {
        listenerList.remove(TreeModelListener.class, l);
    }

    /**
     * Leere Methode, vom Interface TreeModel verlangt
     */
    public void valueForPathChanged(TreePath path, Object newValue) {
    }

    /**
     * Feuert ein Ereignis, welches angibt, dass sich die Wurzel gendert hat.
     */
    private void fireRootChanged() {
        Object[] listeners = listenerList.getListenerList();
        TreeModelEvent e = null;
        for (int i = listeners.length-2; i>=0; i-=2) {
            if (listeners[i] == TreeModelListener.class) {
                if (e == null) {
                    e = new TreeModelEvent(this, new TreePath("\\"));
                }
                ((TreeModelListener)listeners[i+1]).treeStructureChanged(e);
            }
        }
    }

}

